package com.emato.ccnet.biz.req.order.csp;

import com.emato.ccnet.biz.req.CebReqBiz;
import com.emato.ccnet.biz.req.CebReqSendDate;
import com.emato.ccnet.msg.ResponseMessage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.Map;

@Component
public class Ceb311OrderBiz implements CebReqBiz {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb311OrderBiz.class);
    private Map<String, Object> param = Collections.EMPTY_MAP;

    @Autowired
    private Ceb311OrderGetDate ceb311OrderGetDate;

    @Autowired
    private Ceb311OrderAddDate ceb311OrderAddDate;

    @Autowired
    private Ceb311SendDateRequet ceb311SendDateRequet;

    /**
     * 电子订单正常业务入口
     * @param reqParam
     * @return
     */
    @Override
    public ResponseMessage biz(Map<String, Object> reqParam) {

        if (reqParam.isEmpty()) {
            String info = "【CEB411】请求订单支付数据,入参为空";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //查询电子订单数据
        List list = ceb311OrderGetDate.getData(reqParam);
        if (list.isEmpty()) {
            String info = "【CEB411】没有订单数据, 生成海关支付单报文提前结束";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //添加电子订单数据
        ResponseMessage responseMessage = ceb311OrderAddDate.addDate(list);
        if(!responseMessage.isSuccess()){
            String info = "生成海关支付单初始化失败" + responseMessage.getMsg();
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //发送数据请求并响应
        responseMessage = ceb311SendDateRequet.sendDate();
        if(!responseMessage.isSuccess()){
            String info = "发送请求响应失败"+responseMessage.getMsg();
            LOGGER.error("--- {}", responseMessage.getMsg());
            return ResponseMessage.failed(info);
        }

        return ResponseMessage.success();
    }

    public Map<String, Object> getParam() {
        return param;
    }

    public void setParam(Map<String, Object> param) {
        this.param = param;
    }

}
