package com.emato.ccnet.biz.req.order.csp;

import com.emato.ccnet.biz.req.CebReqSendDate;
import com.emato.ccnet.config.Cus;
import com.emato.ccnet.contants.Contants;
import com.emato.ccnet.dto.OrderRequest;
import com.emato.ccnet.domain.ccnet.CusOrder;
import com.emato.ccnet.domain.ccnet.CusOrderGoods;
import com.emato.ccnet.domain.ccnet.CusOrderStatus;
import com.emato.ccnet.message.Result;
import com.emato.ccnet.msg.ResponseMessage;
import com.emato.ccnet.service.ccent.CusOrderGoodsService;
import com.emato.ccnet.service.ccent.CusOrderService;
import com.emato.ccnet.service.ccent.CusOrderStatusService;
import com.emato.ccnet.util.JacksonUtil;
import com.emato.ccnet.util.OkHttpUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import okhttp3.Request;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class Ceb311SendDateRequet implements CebReqSendDate {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb311SendDateRequet.class);

    public static final String CUS_ORDER_EXCLUDE = "cusOrderSn,cusOrderGoodsSn,merchOrderSn,merchSn,merchName,platSn,platName,thirdPartyMerchCode,thirdPartyMerchName,placeOrderlTime,createrSn,createTime,moderSn,modTime,tstm,exField,exField2,exField3,exField4,exField5";

    @Autowired
    private CusOrderService cusOrderService;

    @Autowired
    private Ceb311SendDateResponse ceb311SendDateResponse;

    @Autowired
    private CusOrderGoodsService cusOrderGoodsService;

    @Autowired
    private Ceb311SendDateAfter ceb311SendDateAfter;

    @Autowired
    private CusOrderStatusService cusOrderStatusService;

    @Autowired
    private Cus cus;

    /**
     * 发送请求
     * @return
     */
    @Override
    public ResponseMessage sendDate() {
        //查找当前ccnet中的所有电子订单
        List<CusOrder> cusOrders = cusOrderService.findAllCusOrder();
        if(cusOrders == null){
            String info = "查找当前ccnet中的所有电子订单失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //发送给第三方的请求回调
        ResponseMessage orderSumbit = buildRequest(cusOrders);
        if(!orderSumbit.isSuccess()){
            String info = "查找当前ccnet中的所有电子订单失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }
        return ResponseMessage.success();
    }

    /**
     * 发送第三方请求和回调
     * @param cusOrders
     * @return
     */
    private ResponseMessage buildRequest(List<CusOrder> cusOrders) {

        List<String> sucList = new ArrayList<>();
        List<String> failedList = new ArrayList<>();
        for (CusOrder cusOrder : cusOrders){

            //更新电子订单数据为申报中
            ResponseMessage updateCusOrderStatus = updateCusOrderStatus(cusOrder.getOrderSn());
            if(!updateCusOrderStatus.isSuccess()){
                failedList.add(cusOrder.getOrderSn());
                String info = "更新电子订单数据为申报中失败" + updateCusOrderStatus.getMsg();
                LOGGER.error("--- {}", info);
                continue;
            }

            //数据组装
            OrderRequest orderRequest = assembleData(cusOrder);
            if(orderRequest == null){
                failedList.add(orderRequest.getOrderSn());
                String info = "数据组装失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            //构造请求报文
            Request request = buildRequestDate(orderRequest);
            if(request == null){
                failedList.add(orderRequest.getOrderSn());
                String info = "构造请求报文失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            //发送请求报文，并同步回调响应
            String response = null;
            try {
                response = OkHttpUtil.post(request);
            } catch (IOException e) {
                e.printStackTrace();
            }

            LOGGER.info("response date:"+response);

            //解析响应报文并处理本地业务逻辑
            ResponseMessage responseDis = ceb311SendDateResponse.addResponse(response,orderRequest.getOrderSn());
            if(!responseDis.isSuccess()){
                failedList.add(orderRequest.getOrderSn());
                String info = "解析响应报文并处理本地业务逻辑失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            sucList.add(orderRequest.getOrderSn());
        }

        LOGGER.info("--- 【CEB411】【成功】订单数:{}, 订单: {}", sucList.size(), Joiner.on(",").skipNulls().join(sucList.toArray()));
        LOGGER.info("--- 【CEB411】【失败】订单数:{}, 订单: {}", failedList.size(), Joiner.on(",").skipNulls().join(failedList.toArray()));

        ceb311SendDateAfter.afterSend(sucList,failedList);

        return ResponseMessage.success();
    }

    /**
     * 更新电子订单状态表
     * @param orderSn
     * @return
     */
    private ResponseMessage updateCusOrderStatus(String orderSn) {
        //通过编号查找电子订单表
        Map<String, Object> reqParam = ImmutableMap.of("orderSn", orderSn,"sendStatus",Contants.INIT_STATUS);
        CusOrderStatus cusOrderStatus = cusOrderStatusService.findCusOrderStatus(reqParam);
        if(cusOrderStatus == null){
            return ResponseMessage.failed("该电子订单不存在");
        }

        //更新请求报文状态:请求申报中
        cusOrderStatus.setModerSn("");
        cusOrderStatus.setModTime(new Date());
        cusOrderStatus.setSendStatus(Contants.DECLARATION_STATUS);
        int cusOrderSta = cusOrderStatusService.updateCusOrderStatus(cusOrderStatus);
        if(cusOrderSta != 1){
            return ResponseMessage.failed("该电子订单更新失败");
        }
        return ResponseMessage.success();
    }

    /**
     * 构造请求参数
     * @param orderRequest
     * @return
     */
    private Request buildRequestDate(OrderRequest orderRequest) {
        //屏蔽不需要的字段
        JacksonUtil.filter(CusOrderGoods.class,null,CUS_ORDER_EXCLUDE);
        List list = Lists.newArrayList();
        list.add(orderRequest);
        Result result = Result.success(list);
        String requestData = null;
        try {
            requestData = JacksonUtil.toJson(result);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        LOGGER.info("request Data :" + requestData);
        //构造请求报文
        Request request = OkHttpUtil.buildRequest(cus.getOrder().get("order-url"),requestData);

        return request;
    }

    /**
     * 组装json中所要的数据
     * @param cusOrder
     * @return
     */
    private OrderRequest assembleData(CusOrder cusOrder) {

        // 发送电子订单的集合
        OrderRequest orderRequest = new OrderRequest(cusOrder);

        //通过电子订单内部编号查找海关电子订单商品
        List<CusOrderGoods> cusOrderGoodes = cusOrderGoodsService.findCusOrderGoodsByCusOrderSn(cusOrder.getCusOrderSn());
        if (cusOrderGoodes == null || cusOrderGoodes.size() <= 0) {
            LOGGER.error("通过电子订单内部编号查找海关电子订单商品失败，电子订单内部编号："+cusOrder.getCusOrderSn());
            return null;
        }

        orderRequest.setOrderList(cusOrderGoodes);

        return orderRequest;
    }

}
